/*------------------------------------------------------\
|                                                       |
|  Title   :   Structures and fonctions for BDD         |
|                                                       |
|  Date    :            12.02.96                        |
|                                                       |
|  Author  :        Jacomme Ludovic                     |
|                                                       |
\------------------------------------------------------*/

# ifndef BDD_102_H
# define BDD_102_H

/*------------------------------------------------------\
|                                                       |
|                      Constants                        |
|                                                       |
\------------------------------------------------------*/

# define BDD_OPER_RESTRICT    ((long)0x00010000)
# define BDD_OPER_COMPOSE     ((long)0x00020000)
# define BDD_OPER_SIMP_DC_ON  ((long)0x00040000)
# define BDD_OPER_SIMP_DC_OFF ((long)0x00080000)
# define BDD_OPER_INTERSECT   ((long)0x00100000)
# define BDD_OPER_COFACTOR    ((long)0x00200000)
# define BDD_OPER_SUBSTITUTE  ((long)0x00400000)
# define BDD_OPER_EXIST       ((long)0x00800000)
# define BDD_OPER_EXIST_MISS  ((long)0x01000000)
# define BDD_OPER_REL_PROD    ((long)0x02000000)

# define BDD_OPER_MASK        ((long)0x01FF0000)

/*------------------------------------------------------\
|                                                       |
|                       Bdd Hash Node                   |
|                                                       |
\------------------------------------------------------*/

# define BDD_HASH_NODE_EMPTY   ((bddhnode)0)
# define BDD_HASH_NODE_DELETED ((bddhnode)1)

/*------------------------------------------------------\
|                                                       |
|                        Bdd Index                      |
|                                                       |
\------------------------------------------------------*/

# define BDD_INDEX_ZERO   ((bddindex) 0)
# define BDD_INDEX_ONE    ((bddindex) 1)
# define BDD_INDEX_MIN    ((bddindex) 2)

# define BDD_MAX_INDEX    ((bddindex)-1)

/*------------------------------------------------------\
|                                                       |
|                        Bdd Short                      |
|                                                       |
\------------------------------------------------------*/

# define BDD_MAX_SHORT    ((bddshort)-1)

/*------------------------------------------------------\
|                                                       |
|                        Bdd Variable                   |
|                                                       |
\------------------------------------------------------*/

# define BDD_MAX_VAR  ( ((bddvar)-1) - BDD_INDEX_MIN )
# define BDD_MIN_VAR    ((bddvar)-1)

/*------------------------------------------------------\
|                                                       |
|                     Bdd Reference                     |
|                                                       |
\------------------------------------------------------*/

# define BDD_MAX_REF     ((bddref)-1)

/*------------------------------------------------------\
|                                                       |
|                       Bdd Mark                        |
|                                                       |
\------------------------------------------------------*/

# define BDD_MARK_NUM_MASK      (bddmark)( 0x0001 )
# define BDD_MAX_MARK           (bddmark)( -1     )

/*------------------------------------------------------\
|                                                       |
|                       Support Type                    |
|                                                       |
\------------------------------------------------------*/

# define BDD_SUPPORT_CHAIN     0
# define BDD_SUPPORT_PTYPE     1

/*------------------------------------------------------\
|                                                       |
|                     Add Input Mode                    |
|                                                       |
\------------------------------------------------------*/

# define BDD_IN_MODE_FIRST      0x00
# define BDD_IN_MODE_LAST       0x01
# define BDD_IN_MODE_BEFORE     0x02
# define BDD_IN_MODE_AFTER      0x03
# define BDD_IN_MODE_IMPOSE     0x04

/*------------------------------------------------------\
|                                                       |
|                     Add Auxiliar Mode                 |
|                                                       |
\------------------------------------------------------*/

# define BDD_AUX_MODE_SINGLE     0x00
# define BDD_AUX_MODE_GLOBAL     0x01

/*------------------------------------------------------\
|                                                       |
|                      User Function Type               |
|                                                       |
\------------------------------------------------------*/

# define BDD_FUNC_RESET_HASH_OPER    0

/*------------------------------------------------------\
|                                                       |
|                        Macros                         |
|                                                       |
\------------------------------------------------------*/
/*------------------------------------------------------\
|                                                       |
|                    Bdd Local System                   |
|                                                       |
\------------------------------------------------------*/

# define setbddlocalsystem( S ) \
                                \
  do { if ( (S) ) BddLocalSystem = (S); } while (0)

/*------------------------------------------------------\
|                                                       |
|                    Bdd Local Circuit                  |
|                                                       |
\------------------------------------------------------*/

# define setbddlocalcircuit( C ) \
                                \
  do { if ( (C) ) BddLocalCircuit = (C); } while (0)

/*------------------------------------------------------\
|                                                       |
|                         Type                          |
|                                                       |
\------------------------------------------------------*/
/*------------------------------------------------------\
|                                                       |
|                 Index Ref Mark and Var                |
|                                                       |
\------------------------------------------------------*/

  typedef unsigned short bddindex;
  typedef unsigned short bddref;
  typedef unsigned short bddmark;
  typedef unsigned short bddvar;
  typedef unsigned short bddshort;

/*------------------------------------------------------\
|                                                       |
|                       Bdd Alloc Info                  |
|                                                       |
\------------------------------------------------------*/

  typedef struct bddallocinfo 
  {
    long  CIRCUIT;
    long  SYSTEM;
    long  BLOCK;
    long  NODE_BLOCK;
    long  HASH_NODE_TABLE;
    long  HASH_NODE;
    long  HASH_OPER_TABLE;
    long  HASH_OPER;
    long  VAR_TREE;
    long  VAR_CHILD;
    long  VAR;
    long  VAR_NODE;
    long  INDEX_NODE;
    long  INDEX;
    long  ASSOC;
    long  ASSOC_NODE;
    long  NAME_IN;
    long  INDEX_IN;
    long  HEATH;
    long  USER_FUNC;

  } bddallocinfo;

/*------------------------------------------------------\
|                                                       |
|                       Bdd User Function               |
|                                                       |
\------------------------------------------------------*/

  typedef struct bdduserfunc
  {
    struct bdduserfunc *NEXT;
    void              (*FUNC)();
    void               *DATA;
    long                TYPE;

  } bdduserfunc;

/*------------------------------------------------------\
|                                                       |
|                       Bdd Node                        |
|                                                       |
\------------------------------------------------------*/

  typedef struct bddnode
  {
    struct bddnode *HIGH;
    struct bddnode *LOW;
    bddindex        INDEX;
    bddmark         MARK;
    bddref          REF_EXT;
    bddref          REF_INT;

  } bddnode;

/*------------------------------------------------------\
|                                                       |
|                       Bdd Hash Node                   |
|                                                       |
\------------------------------------------------------*/

  typedef bddnode *bddhnode;

/*------------------------------------------------------\
|                                                       |
|                       Bdd Block                       |
|                                                       |
\------------------------------------------------------*/

  typedef struct bddblock
  {
    struct bddblock *NEXT;
    struct bddnode  *NODE;
    long             NODE_FREE;

  } bddblock;

/*------------------------------------------------------\
|                                                       |
|                    Bdd Hash Node Table                |
|                                                       |
\------------------------------------------------------*/

  typedef struct bddhnodetable
  {
    bddhnode *TABLE;
    long      TABLE_SIZE;
    long      NUMBER_NODE;
    long      NUMBER_ADD;
    long      NUMBER_SCAN;
    long      NUMBER_STRETCH;
    long      NUMBER_RESIZE;
    
  } bddhnodetable;

/*------------------------------------------------------\
|                                                       |
|                       Bdd Hash Oper                   |
|                                                       |
\------------------------------------------------------*/

  typedef struct bddhoper
  {
    bddnode *LEFT;
    bddnode *RIGHT;
    bddnode *FATHER;
    long     OPERATOR;

  } bddhoper;

/*------------------------------------------------------\
|                                                       |
|                    Bdd Hash Oper Table                |
|                                                       |
\------------------------------------------------------*/

  typedef struct bddhopertable
  {
    bddhoper *TABLE;
    long      TABLE_SIZE;
    long      TABLE_FREEZE;
    long      NUMBER_OPER;
    long      NUMBER_ADD;
    long      NUMBER_HIT;
    long      NUMBER_MISS;
    long      NUMBER_RESET;

  } bddhopertable;

/*------------------------------------------------------\
|                                                       |
|                   Bdd Variable Tree                   |
|                                                       |
\------------------------------------------------------*/

  typedef struct bddvartree
  {
    struct bddvartree **CHILD;
    bddvar              NUMBER_CHILD;
    bddvar              FIRST;
    bddvar              LAST;
    bddvar              REORDER;

  } bddvartree;

/*------------------------------------------------------\
|                                                       |
|                   Bdd Variable Child                  |
|                                                       |
\------------------------------------------------------*/

  typedef bddvartree *bddvarchild;

/*------------------------------------------------------\
|                                                       |
|                   Bdd Variable Node                   |
|                                                       |
\------------------------------------------------------*/

  typedef bddnode    *bddvarnode;

/*------------------------------------------------------\
|                                                       |
|                    Bdd Index Node                     |
|                                                       |
\------------------------------------------------------*/

  typedef bddhnodetable *bddindexnode;

/*------------------------------------------------------\
|                                                       |
|                     Bdd Assoc Node                    |
|                                                       |
\------------------------------------------------------*/

  typedef bddnode  *bddassocnode;

/*------------------------------------------------------\
|                                                       |
|                       Bdd Assoc                       |
|                                                       |
\------------------------------------------------------*/

  typedef struct bddassoc
  {
    struct bddassoc *NEXT;
    bddassocnode    *ASSOC_NODE;
    bddvar           FIRST;
    bddvar           LAST;
    bddshort         IDENT;
    bddshort         CACHE;

  } bddassoc;

/*------------------------------------------------------\
|                                                       |
|                       Bdd System                      |
|                                                       |
\------------------------------------------------------*/

  typedef struct bddsystem
  {
    bddblock       *BLOCK;
    bddnode        *NODE_FREE;
    long            NUMBER_FREE;

    bddindexnode   *INDEX_NODE;
    bddvarnode     *VAR_NODE;
    bddindex       *VAR_TO_INDEX;
    bddvar         *INDEX_TO_VAR;


    long            NUMBER_NODE;
    long            MAX_NODE;
    long            TOP_NODE;

    long            NUMBER_VAR;
    long            MAX_VAR;

    long            NUMBER_INDEX;
    long            MAX_INDEX;

    bddassoc       *ASSOC;
    unsigned char   ASSOC_RESET;

    bddvartree     *VAR_TREE;
    bddhopertable  *HASH_OPER;

    bddnode        *ONE;
    bddnode        *ZERO;

    long            REORDER;
    void          (*REORDER_FUNC)();
    long            REORDER_LOW;
    long            REORDER_LIMIT;
    long            REORDER_RATIO;

    bdduserfunc    *USER_FUNC;

    long            VAR_MODEL;
    long            OPER_MODEL;

  } bddsystem;

/*------------------------------------------------------\
|                                                       |
|                     Bdd Name Input                    |
|                                                       |
\------------------------------------------------------*/

  typedef char *bddnamein;

/*------------------------------------------------------\
|                                                       |
|                       Bdd Circuit                     |
|                                                       |
\------------------------------------------------------*/

  typedef struct bddcircuit 
  {
    char       *NAME;

    authtable  *HASH_IN;
    authtable  *HASH_OUT;

    bddnamein  *NAME_IN;
    long        NAME_IN_SIZE;

    bddindex   *INDEX_IN;
    long        INDEX_IN_SIZE;

    long        NUMBER_NAME_IN;
    long        NUMBER_NAME_OUT;

    bddsystem  *BDD_SYSTEM;

  } bddcircuit;

/*------------------------------------------------------\
|                                                       |
|                       Bdd Heath                       |
|                                                       |
\------------------------------------------------------*/

  typedef struct bddheath
  {
    struct bddheath *NEXT;
    struct bddnode  *VAR;
    struct bddnode  *MINUS;
    struct bddnode  *PLUS;

  } bddheath;

/*------------------------------------------------------\
|                                                       |
|                     Global Variables                  |
|                                                       |
\------------------------------------------------------*/

  extern long BDD_HNODE_STRETCH_FACTOR;
  extern long BDD_HNODE_RESIZE_FACTOR;
  extern long BDD_HNODE_MIN_FILL_FACTOR;
  extern long BDD_HNODE_MAX_SCAN;

  extern bddsystem    *BddLocalSystem;
  extern bddcircuit   *BddLocalCircuit;
  extern bddallocinfo  BddAllocInfo;

/*------------------------------------------------------\
|                                                       |
|                        Functions                      |
|                                                       |
\------------------------------------------------------*/
/*------------------------------------------------------\
|                                                       |
|                       Env Functions                   |
|                                                       |
\------------------------------------------------------*/

  extern void           bddenv();

/*------------------------------------------------------\
|                                                       |
|                       Alloc Functions                 |
|                                                       |
\------------------------------------------------------*/

  extern bddcircuit    *allocbddcircuit();
  extern bddsystem     *allocbddsystem();

  extern bddblock      *allocbddblock();
  extern bddnode       *allocbddnodeblock();
  extern bddhnodetable *allocbddhnodetable();
  extern bddhnode      *allocbddhnode();
  extern bddhopertable *allocbddhopertable();
  extern bddhoper      *allocbddhoper();

  extern bddvartree    *allocbddvartree();
  extern bddvarchild   *allocbddvarchild();
  extern bddvar        *allocbddvar();
  extern bddvarnode    *allocbddvarnode();
  extern bddindexnode  *allocbddindexnode();
  extern bddindex      *allocbddindex();
  extern bddnamein     *allocbddnamein();
  extern bddindex      *allocbddindexin();

  extern bddassoc      *allocbddassoc();
  extern bddassocnode  *allocbddassocnode();

  extern bddheath      *allocbddheath();
  extern bdduserfunc   *allocbdduserfunc();

  extern void           viewbddallocinfo();

/*------------------------------------------------------\
|                                                       |
|                      Resize Functions                 |
|                                                       |
\------------------------------------------------------*/

  extern bddvarchild   *resizebddvarchild();
  extern bddvar        *resizebddvar();
  extern bddvarnode    *resizebddvarnode();
  extern bddindexnode  *resizebddindexnode();
  extern bddindex      *resizebddindex();
  extern bddassocnode  *resizebddassocnode();
  extern bddnamein     *resizebddnamein();
  extern bddindex      *resizebddindexin();

/*------------------------------------------------------\
|                                                       |
|                       Free Functions                  |
|                                                       |
\------------------------------------------------------*/

  extern void  freebddcircuit();
  extern void  freebddsystem();
  extern void  freebddblock();
  extern void  freebddnodeblock();
  extern void  freebddhnodetable();
  extern void  freebddhnode();
  extern void  freebddhopertable();
  extern void  freebddhoper();
  extern void  freebddvartree();
  extern void  freebddvarchild();
  extern void  freebddvar();
  extern void  freebddvarnode();
  extern void  freebddindex();
  extern void  freebddindexnode();
  extern void  freebddnamein();
  extern void  freebddindexin();
  extern void  freebddassoc();
  extern void  freebddassocnode();
  extern void  freebddheath();
  extern void  freebdduserfunc();

/*------------------------------------------------------\
|                                                       |
|                     Circuit Functions                 |
|                                                       |
\------------------------------------------------------*/

  extern bddcircuit *createbddcircuit();
  extern void        resetbddcircuit();
  extern void        destroybddcircuit();
  extern void        viewbddcircuit();

  extern bddnode    *searchbddcircuitin();
  extern bddnode    *addbddcircuitin();
  extern bddnode    *addbddcircuitaux();

  extern bddnode    *searchbddcircuitout();
  extern bddnode    *addbddcircuitout();
  extern int         delbddcircuitout();

  extern bddnode    *addbddcircuitabl();
  extern ablexpr    *convertbddcircuitabl();
  extern ablexpr    *convertbddcircuitsumabl();

/*------------------------------------------------------\
|                                                       |
|                      System Functions                 |
|                                                       |
\------------------------------------------------------*/

  extern bddsystem  *createbddsystem();
  extern void        resetbddsystem();
  extern void        destroybddsystem();
  extern void        viewbddsystem();
  extern void        viewbddsysteminfo();

/*------------------------------------------------------\
|                                                       |
|                      Block  Functions                 |
|                                                       |
\------------------------------------------------------*/

  extern bddblock   *createbddblock();
  extern void        resetbddblock();
  extern void        destroybddblock();
  extern void        viewbddblock();

  extern bddblock   *addbddblock();

/*------------------------------------------------------\
|                                                       |
|                  Variable Tree Functions              |
|                                                       |
\------------------------------------------------------*/

  extern bddvartree *createbddvartree();
  extern void        resetbddvartree();
  extern void        destroybddvartree();
  extern void        viewbddvartree();

  extern bddvar      searchbddvartree();
  extern void        deltabddvartree();
  extern bddvartree *shiftbddvartree();
  extern bddvartree *addbddvartree();

/*------------------------------------------------------\
|                                                       |
|                       User Functions                  |
|                                                       |
\------------------------------------------------------*/

  extern bdduserfunc *addbdduserfunc();
  extern void         execbdduserfunc();
  extern int          delbdduserfunc();
  extern void         destroybdduserfunc();

/*------------------------------------------------------\
|                                                       |
|                    Variable Functions                 |
|                                                       |
\------------------------------------------------------*/

  extern bddindex  newbddvar();
  extern bddnode  *addbddvar();
  extern bddnode  *addbddvarfirst();
  extern bddnode  *addbddvarlast();
  extern bddnode  *addbddvarbefore();
  extern bddnode  *addbddvarafter();

  extern void      swapbddvar();
  extern void      sweepbddvar();

  extern bddindex  getbddvarindex();
  extern bddvar    getbddvarbyindex();
  extern bddnode  *getbddvarnode();
  extern bddnode  *getbddvarnodebyindex();

/*------------------------------------------------------\
|                                                       |
|                Auxiliary Variable Functions           |
|                                                       |
\------------------------------------------------------*/

  extern bddnode  *addbddvarauxsingle();
  extern bddnode  *addbddvarauxglobal();
  extern bddnode  *addbddvarauxuser();

/*------------------------------------------------------\
|                                                       |
|                     Node  Functions                   |
|                                                       |
\------------------------------------------------------*/

  extern bddnode    *addbddnode();
  extern void        delbddnode();
  extern void        viewbddnode();

  extern chain_list *addbddnodelist();
  extern void        delbddnodelist();

/*------------------------------------------------------\
|                                                       |
|                   Reference Functions                 |
|                                                       |
\------------------------------------------------------*/

  extern bddnode  *incbddrefext();
  extern bddnode  *decbddrefext();
  extern bddnode  *incbddrefint();
  extern bddnode  *decbddrefint();

  extern bddnode  *setbddrefext();
  extern bddnode  *unsetbddrefext();

  extern void      updatebddref();

  extern void      clearbddsystemrefext();
  extern void      clearbddsystemrefint();
  extern void      clearbddsystemref();

/*------------------------------------------------------\
|                                                       |
|                   Association  Functions              |
|                                                       |
\------------------------------------------------------*/

  extern bddassoc  *addbddassoc();
  extern int        delbddassoc();
  extern void       viewbddassoc();

  extern void       resetbddassoc();
  extern void       destroybddassoc();

  extern bddassoc  *addbddnodeassoc();
  extern int        delbddnodeassoc();

/*------------------------------------------------------\
|                                                       |
|                      Mark Functions                   |
|                                                       |
\------------------------------------------------------*/

  extern long   markbddnode();
  extern long   unmarkbddnode();
  extern long   getbddnodenum();
  extern long   getbddnodesize();

/*------------------------------------------------------\
|                                                       |
|                     Check Functions                   |
|                                                       |
\------------------------------------------------------*/

  extern int    checkbddindex();
  extern int    checkbddvar();
  extern int    checkbddoper();
  extern int    checkbddassoc();
  extern int    checkbddmaxnode();

/*------------------------------------------------------\
|                                                       |
|                   Get Hash Functions                  |
|                                                       |
\------------------------------------------------------*/

  extern long       getbddhnodesize();
  extern long       getbddhnodekey();
  extern long       getbddhnodeindex();

  extern long       getbddhopersize();
  extern long       getbddhoperkey();

/*------------------------------------------------------\
|                                                       |
|                 Check Hash Functions                  |
|                                                       |
\------------------------------------------------------*/

  extern int        checkbddhnode();

/*------------------------------------------------------\
|                                                       |
|                   Set Hash Functions                  |
|                                                       |
\------------------------------------------------------*/

  extern void       setbddhnodefunc();
  extern void       setbddhoperfunc();

/*------------------------------------------------------\
|                                                       |
|                   Hash Table Functions                |
|                                                       |
\------------------------------------------------------*/

  extern bddhnodetable *createbddhnodetable();
  extern void           stretchbddhnodetable();
  extern void           destroybddhnodetable();
  extern void           resetbddhnodetable();
  extern void           viewbddhnodetable();

  extern bddhopertable *createbddhopertable();
  extern void           destroybddhopertable();
  extern void           resetbddhopertable();
  extern void           viewbddhopertable();

/*------------------------------------------------------\
|                                                       |
|                    Hash Node Functions                |
|                                                       |
\------------------------------------------------------*/

  extern bddnode   *addbddhnode();
  extern int        delbddhnode();
  extern void       viewbddhnode();

/*------------------------------------------------------\
|                                                       |
|                    Hash Oper Functions                |
|                                                       |
\------------------------------------------------------*/

  extern bddhoper  *addbddhoper();
  extern bddhoper  *searchbddhoper();
  extern void       viewbddhoper();

/*------------------------------------------------------\
|                                                       |
|               View Index and Variable Nodes           |
|                                                       |
\------------------------------------------------------*/

  extern void   viewbddindexnode();
  extern void   viewbddvarnode();

/*------------------------------------------------------\
|                                                       |
|                      Apply Functions                  |
|                                                       |
\------------------------------------------------------*/

  extern bddnode *applybddnode();
  extern bddnode *applybddnodenot();
  extern bddnode *applybddnodeite();
  extern bddnode *applybddnodeterm();
  extern bddnode *applybddnodelist();

/*------------------------------------------------------\
|                                                       |
|                    Support Functions                  |
|                                                       |
\------------------------------------------------------*/

  extern void *getbddnodesupport();
  extern int   isbddvarinsupport();

/*------------------------------------------------------\
|                                                       |
|                    Compose Functions                  |
|                                                       |
\------------------------------------------------------*/

  extern bddnode *restrictbddnode();
  extern bddnode *composebddnode();

/*------------------------------------------------------\
|                                                       |
|                  Substitute Functions                 |
|                                                       |
\------------------------------------------------------*/

  extern bddnode *substbddnodeassoc();

/*------------------------------------------------------\
|                                                       |
|                      Exist Functions                  |
|                                                       |
\------------------------------------------------------*/

  extern bddnode *existbddnodeassocon();
  extern bddnode *existbddnodeassocoff();

/*------------------------------------------------------\
|                                                       |
|                       Miss Functions                  |
|                                                       |
\------------------------------------------------------*/

  extern bddnode *missbddnodeassocon();
  extern bddnode *missbddnodeassocoff();

/*------------------------------------------------------\
|                                                       |
|                  Relational Product Functions         |
|                                                       |
\------------------------------------------------------*/

  extern bddnode *relprodbddnodeassoc();

/*------------------------------------------------------\
|                                                       |
|               Intersect & Imply Functions             |
|                                                       |
\------------------------------------------------------*/

  extern bddnode *implybddnode();
  extern bddnode *intersectbddnode();

/*------------------------------------------------------\
|                                                       |
|                 Fraction Functions                    |
|                                                       |
\------------------------------------------------------*/

  extern double  fractionbddnode();

/*------------------------------------------------------\
|                                                       |
|                 Satisfy Functions                     |
|                                                       |
\------------------------------------------------------*/

  extern bddnode *satisfybddnode();
  extern bddnode *satisfybddnodeassoc();

/*------------------------------------------------------\
|                                                       |
|                    Cofactor Functions                 |
|                                                       |
\------------------------------------------------------*/

  extern bddnode *cofactorbddnode();

/*------------------------------------------------------\
|                                                       |
|                      Heath Functions                  |
|                                                       |
\------------------------------------------------------*/

  extern bddheath *getbddheath();
  extern bddheath *getbddheathvar();
  extern void      delbddheath();

/*------------------------------------------------------\
|                                                       |
|                    Simplify Functions                 |
|                                                       |
\------------------------------------------------------*/

  extern bddnode *simpbddnodedcon();
  extern bddnode *simpbddnodedcoff();

/*------------------------------------------------------\
|                                                       |
|                   Conversion Functions                |
|                                                       |
\------------------------------------------------------*/

  extern ablexpr    *convertbddnodeabl();
  extern ablexpr    *convertbddnodemuxabl();
  extern ablexpr    *convertbddnodesumabl();
  extern ablexpr    *convertbddindexabl();

/*------------------------------------------------------\
|                                                       |
|                Garbage Collect Functions              |
|                                                       |
\------------------------------------------------------*/

  extern void   garbagebddsystem();
  extern void   sweepbddsystem();

/*------------------------------------------------------\
|                                                       |
|                   Reorder Functions                   |
|                                                       |
\------------------------------------------------------*/

  extern int    reorderbddvartreewindow2();
  extern int    reorderbddvartreewindow3();
  extern void   reorderbddsystemsimple();
  extern void   reorderbddsystemwindow();
  extern void   reorderbddsystemtop();

  extern void   reorderbddsystemdynamic();

/*------------------------------------------------------\
|                                                       |
|                    Test Functions                     |
|                                                       |
\------------------------------------------------------*/

  extern int   testbddcircuit();

/*------------------------------------------------------\
|                                                       |
|                    Dump Functions                     |
|                                                       |
\------------------------------------------------------*/

  extern bddcircuit *undumpbddcircuit();
  extern void        dumpbddcircuit();

/*------------------------------------------------------\
|                                                       |
|             Log Compatibility Functions               |
|                                                       |
\------------------------------------------------------*/

   extern bddnode *getlogbddnodeone();
   extern bddnode *getlogbddnodezero();
   extern bddnode *getlogbddnodeterm();
   extern bddnode *getlogbddnodenot();
   extern bddnode *applylogbddnode();
   extern ablexpr *convertlogbddnode();
   extern short    addlogbddcircuitin();


# endif 
